@@ -5,17 +5,12 @@ use warnings;
 
 use DBIx::Class::ResultSet;
 
-use Carp qw/croak/;
-
 use base qw/DBIx::Class/;
-__PACKAGE__->load_components(qw/ResultSource AccessorGroup/);
+__PACKAGE__->load_components(qw/ResultSource/);
 
-__PACKAGE__->mk_group_accessors('simple' =>
-  qw/_columns _primaries name resultset_class result_class schema/);
+=head1 NAME
 
-=head1 NAME 
-
-DBIx::Class::Table - Table object
+DBIx::Class::ResultSource::Table - Table object
 
 =head1 SYNOPSIS
 
