@@ -14,7 +14,7 @@ sub _rebless {
         my ($major, $minor, $patchlevel) = split(/\./, $version);
 
         # Default driver
-        my $class = $major >= 8
+        my $class = $major <= 8
           ? 'DBIx::Class::Storage::DBI::Oracle::WhereJoins'
           : 'DBIx::Class::Storage::DBI::Oracle::Generic';
 
@@ -25,6 +25,19 @@ sub _rebless {
     }
 }
 
+sub _svp_begin {
+    my ($self, $name) = @_;
+ 
+    $self->dbh->do("SAVEPOINT $name");
+}
+
+# Would've implemented _svp_release here, but Oracle doesn't support it.
+
+sub _svp_rollback {
+    my ($self, $name) = @_;
+
+    $self->dbh->do("ROLLBACK TO SAVEPOINT $name")
+}
 
 1;
 
@@ -40,8 +53,16 @@ DBIx::Class::Storage::DBI::Oracle - Base class for Oracle driver
 =head1 DESCRIPTION
 
 This class simply provides a mechanism for discovering and loading a sub-class
-for a specific version Oracle backend.  It should be transparent to the user.
+for a specific version Oracle backend. It should be transparent to the user.
 
+For Oracle major versions <= 8 it loads the ::Oracle::WhereJoins subclass,
+which unrolls the ANSI join style DBIC normally generates into entries in
+the WHERE clause for compatibility purposes. To force usage of this version
+no matter the database version, add
+
+  __PACKAGE__->storage_type('::DBI::Oracle::WhereJoins');
+
+to your Schema class.
 
 =head1 AUTHORS
 
