@@ -7,8 +7,10 @@ no warnings qw/qw/;
 
 __PACKAGE__->load_classes(qw/
   Artist
+  SequenceTest
   Employee
   CD
+  FileColumn
   Link
   Bookmark
   #dummy
@@ -29,10 +31,21 @@ __PACKAGE__->load_classes(qw/
     'SelfRef',
     'ArtistUndirectedMap',
     'ArtistSourceName',
+    'ArtistSubclass',
     'Producer',
     'CD_to_Producer',
   ),
-  qw/SelfRefAlias TreeLike TwoKeyTreeLike Event/
+  qw/SelfRefAlias TreeLike TwoKeyTreeLike Event EventTZ NoPrimaryKey/,
+  qw/Collection CollectionObject TypedObject/,
+  qw/Owners BooksInLibrary/,
+  qw/ForceForeign/,
+  qw/LongColumns/,
 );
 
+sub sqlt_deploy_hook {
+  my ($self, $sqlt_schema) = @_;
+
+  $sqlt_schema->drop_table('dummy');
+}
+
 1;
